@@ -430,9 +430,19 @@ void print_resource_track(e2fsck_t ctx, const char *desc,
 		log_out(ctx, "%s: ", desc);
 
 #define kbytes(x)	(((unsigned long long)(x) + 1023) / 1024)
-#ifdef HAVE_MALLINFO
+#ifdef HAVE_MALLINFO2
+	if (1) {
+		struct mallinfo2 malloc_info = mallinfo2();
+
+		log_out(ctx, _("Memory used: %lluk/%lluk (%lluk/%lluk), "),
+			kbytes(malloc_info.arena), kbytes(malloc_info.hblkhd),
+			kbytes(malloc_info.uordblks),
+			kbytes(malloc_info.fordblks));
+	} else
+#elif defined HAVE_MALLINFO
 	/* don't use mallinfo() if over 2GB used, since it returns "int" */
-	if ((char *)sbrk(0) - (char *)track->brk_start < 2LL << 30) {
+	if ((unsigned long)((char *)sbrk(0) - (char *)track->brk_start) <
+	    2UL << 30) {
 		struct mallinfo	malloc_info = mallinfo();
 
 		log_out(ctx, _("Memory used: %lluk/%lluk (%lluk/%lluk), "),
@@ -550,29 +560,20 @@ blk64_t get_backup_sb(e2fsck_t ctx, ext2_filsys fs, const char *name,
 	struct ext2_super_block *sb;
 	io_channel		io = NULL;
 	void			*buf = NULL;
-	int			blocksize;
-	blk64_t			superblock, ret_sb = 8193;
+	unsigned int		blocksize = EXT2_MIN_BLOCK_SIZE;
+	int			blocksize_known = 0;
+	blk_t			bpg = 0;
+	blk64_t			ret_sb = 8193;
 
 	if (fs && fs->super) {
-		ret_sb = (fs->super->s_blocks_per_group +
-			  fs->super->s_first_data_block);
-		if (ctx) {
-			ctx->superblock = ret_sb;
-			ctx->blocksize = fs->blocksize;
-		}
-		return ret_sb;
+		blocksize = fs->blocksize;
+		blocksize_known = 1;
+		bpg = fs->super->s_blocks_per_group;
 	}
 
-	if (ctx) {
-		if (ctx->blocksize) {
-			ret_sb = ctx->blocksize * 8;
-			if (ctx->blocksize == 1024)
-				ret_sb++;
-			ctx->superblock = ret_sb;
-			return ret_sb;
-		}
-		ctx->superblock = ret_sb;
-		ctx->blocksize = 1024;
+	if (ctx && ctx->blocksize) {
+		blocksize = ctx->blocksize;
+		blocksize_known = 1;
 	}
 
 	if (!name || !manager)
@@ -585,28 +586,54 @@ blk64_t get_backup_sb(e2fsck_t ctx, ext2_filsys fs, const char *name,
 		goto cleanup;
 	sb = (struct ext2_super_block *) buf;
 
-	for (blocksize = EXT2_MIN_BLOCK_SIZE;
-	     blocksize <= EXT2_MAX_BLOCK_SIZE ; blocksize *= 2) {
-		superblock = blocksize*8;
-		if (blocksize == 1024)
-			superblock++;
-		io_channel_set_blksize(io, blocksize);
-		if (io_channel_read_blk64(io, superblock,
-					-SUPERBLOCK_SIZE, buf))
-			continue;
-#ifdef WORDS_BIGENDIAN
-		if (sb->s_magic == ext2fs_swab16(EXT2_SUPER_MAGIC))
-			ext2fs_swap_super(sb);
-#endif
-		if ((sb->s_magic == EXT2_SUPER_MAGIC) &&
-		    (EXT2_BLOCK_SIZE(sb) == blocksize)) {
-			ret_sb = superblock;
-			if (ctx) {
-				ctx->superblock = superblock;
-				ctx->blocksize = blocksize;
-			}
-			break;
+	for (; blocksize <= EXT2_MAX_BLOCK_SIZE; blocksize *= 2) {
+		dgrp_t grp, three = 1, five = 5, seven = 7;
+		dgrp_t limit;
+		blk_t this_bpg = bpg ? bpg : blocksize * 8;
+		blk64_t	num_blocks;
+
+		if (fs && fs->super) {
+			limit = ext2fs_blocks_count(fs->super) / this_bpg;
+		} else if (ctx && ext2fs_get_device_size2(ctx->filesystem_name,
+							  blocksize,
+							  &num_blocks) == 0) {
+			limit = num_blocks / this_bpg;
+		} else {
+			/* If we can't figure out the device size,
+			 * arbitrarily set a limit which is enough for
+			 * 8 block groups or so...
+			 */
+			limit = 128;
 		}
+
+		io_channel_set_blksize(io, blocksize);
+
+		while ((grp = ext2fs_list_backups(NULL, &three,
+						  &five, &seven)) <= limit) {
+			blk64_t superblock = (blk64_t)grp * this_bpg;
+
+			if (blocksize == 1024)
+				superblock++;
+			if (io_channel_read_blk64(io, superblock,
+						-SUPERBLOCK_SIZE, buf))
+				continue;
+#ifdef WORDS_BIGENDIAN
+			if (sb->s_magic == ext2fs_swab16(EXT2_SUPER_MAGIC))
+				ext2fs_swap_super(sb);
+#endif
+			if ((sb->s_magic == EXT2_SUPER_MAGIC) &&
+			    ((unsigned) EXT2_BLOCK_SIZE(sb) == blocksize)) {
+				ret_sb = superblock;
+				if (ctx) {
+					ctx->superblock = superblock;
+					ctx->blocksize = blocksize;
+				}
+				goto cleanup;
+			}
+		}
+
+		if (blocksize_known)
+			break;
 	}
 
 cleanup:
@@ -853,39 +880,20 @@ errcode_t e2fsck_allocate_subcluster_bitmap(ext2_filsys fs, const char *descr,
 unsigned long long get_memory_size(void)
 {
 #if defined(_SC_PHYS_PAGES)
-# if defined(_SC_PAGESIZE)
-	return (unsigned long long)sysconf(_SC_PHYS_PAGES) *
-	       (unsigned long long)sysconf(_SC_PAGESIZE);
-# elif defined(_SC_PAGE_SIZE)
-	return (unsigned long long)sysconf(_SC_PHYS_PAGES) *
-	       (unsigned long long)sysconf(_SC_PAGE_SIZE);
-# endif
-#elif defined(CTL_HW)
-# if (defined(HW_MEMSIZE) || defined(HW_PHYSMEM64))
-#  define CTL_HW_INT64
-# elif (defined(HW_PHYSMEM) || defined(HW_REALMEM))
-#  define CTL_HW_UINT
-# endif
-	int mib[2];
+	long	phys_pages = sysconf(_SC_PHYS_PAGES);
+	long	pagesize = -1;
 
-	mib[0] = CTL_HW;
-# if defined(HW_MEMSIZE)
-	mib[1] = HW_MEMSIZE;
-# elif defined(HW_PHYSMEM64)
-	mib[1] = HW_PHYSMEM64;
-# elif defined(HW_REALMEM)
-	mib[1] = HW_REALMEM;
-# elif defined(HW_PYSMEM)
-	mib[1] = HW_PHYSMEM;
+# if defined(_SC_PAGESIZE)
+	pagesize = sysconf(_SC_PAGESIZE);
+# elif  defined(_SC_PAGE_SIZE)
+	pagesize = sysconf(_SC_PAGE_SIZE);
 # endif
-# if defined(CTL_HW_INT64)
-	unsigned long long size = 0;
-# elif defined(CTL_HW_UINT)
-	unsigned int size = 0;
-# endif
-	return 0;
+	if (pagesize < 0)
+		pagesize = 4096;
+	if (phys_pages > 0)
+		return (unsigned long long) phys_pages * pagesize;
 #else
 # warning "Don't know how to detect memory on your platform?"
-	return 0;
 #endif
+	return 0;
 }
